package tiny
import java.util.Scanner
import java.io.FileReader

/**
 * A Tiny Calculator Language Compiler
 * 
 * Our compiler translates calculator source code to x86 machine code
 * in stages using multiple internal and external program representations.
 * 
 * External program representations:
 * - Calculator source code
 * - x86 assembly language code
 * - x86 machine code
 * 
 * Internal program representations:
 * - AST.scala
 *     AST: Abstract Syntax Tree
 * - IR.scala
 *     IR: Intermediate Representation/Three-Address Code
 * 
 * Compilation stages:
 * - Parse.scala
 *     Lexing and Parsing: source to AST
 * - Lower.scala
 *     Scope-checking and Lowering: AST to IR
 * - Opt.scala
 *     Optimization: IR to IR
 * - CodeGen.scala
 *     Code Generation: IR to x86 assembly
 * - Link.scala
 *     [External] Assembly and Linking: assembly to machine code executable
 * 
 */
object Compiler {
	def main(args: Array[String]): Unit = {
		try {
			val source = if (args.isEmpty) new Lexer(System.in) else new Lexer(new FileReader(args(0)))

			// 1. PARSE
			// Parse the source code character sequence to an Abstract Syntax Tree.
			val ast = new parser(source).parse().value match { case p: Program => p }
			println("== AST ========")
			println(ast)

			// 2. IRGEN
			// Convert the AST to a Three-Address Code Intermediate Representation.
//			println("== IR ========")
//			val ir = IRGen(ast)
//			for (inst <- ir) println("  " + inst)

			// 3. OPTIMIZE
			// Optimize the IR program.
			// println("== OPT ========")
			// val irOpt = Opt(ir)

			// 4. CODEGEN
			// Translate the TAC into x86 assembly code.
			// println("== ASM ========")
			// val asm = CodeGen(irOpt)
			// println(asm)

			// 5. LINK
			// [External] Assemble the generated code and link a binary executable.
			// println("== BIN ========")
			// Link(asm, if (args.isEmpty) "a.s" else args(0) + ".s", "a.out")
			
  		println("Success.")
		}
		// Commented out for now so you van see where spurious errors originate.
//		catch {
//		  // Die with compiler errors.
//			case e: CompilerError => {
//			  println(e.getClass.getName + ": " + e.getMessage)
//			  println("REJECTED")
//			}
//	  }
	}
}

/** Common basis for various types of compiler errors. */
class CompilerError(msg: String) extends Exception(msg)

